#delimit;

/* version 1.0 */
/* last modified 1/10/02 */   

program define sartpred;

version 6;

   

syntax varlist(min=2) [, corr(real 1) se(real 1) psel(string) pcond(string)];  

if abs(`corr')!= 1 {;  	/* check whether 1 or -1 is selected for the correlation */

   local corr = 1;      /* if other than 1 or -1,the correlation is set to 1 (the default) */

}; 

if `se'!=1 {;      /* checks whether se is 1 */

   local se = 0;   /* if se is not 1 then se is assumed to be 0 (don't report standard errors */

};  



if "`psel'" == "" {;

  local psel = "psel";

};

if "`pcond'" == "" {;

  local pcond = "pcond";

};



tokenize `varlist';

local Y3 "`1'";

macro shift 1;

local X "`*'";

   

quietly {;

   tempvar coef xgsel xbout check coefg coefb VCV VCVp;

   matrix `coef' = e(b);    			/* creates vector of coefficients */

   matrix par = colsof(coef);   		/* counts total number of coefficients */

   local param = par[1,1];

   local p2 = `param';

   local p = `p2'/2;        			/* counts number of covariates */

   local pm1 = `p'-1;

   local p1 = `p'+1;

   local pm2 = `p2'-1;

   matrix `coefg' = `coef'[1,1..`p'];  		/* creates a vector of selection coefficients */

   matrix `coefb' = `coef'[1,`p1'..`p2'];  	/* creates a vector of outcome coefficients */

   matrix score `xgsel' = `coefg';         	/* creates x*gamma */

   matrix score `xbout' = `coefb';         	/* creates x*beta */

   mat `VCV' = e(V);                       	/* creates variance covariance matrix */

   mat `VCVp' = `VCV'[1..`p',1..`p'];           /* creates selection coef var-cov matrix*/

   

   gen `psel' = norm(`xgsel');			/* CDF for x*gamma which is the probability of selection */

   

   if `corr' == 1 {              		/* Calculating the cond prob if corr = 1 */ 

      gen `pcond' = norm(`xbout')/norm(`xgsel') if `xgsel' > `xbout';

      replace `pcond' = 1 if `xgsel' <= `xbout';

   };

   

   if `corr' == -1 {;      			/* Calculating cond prob if corr = -1 */

     gen `pcond' = (norm(`xgsel')-norm(-`xbout'))/norm(`xgsel') if

     `xgsel'+`xbout' > 0;

     replace `pcond' = 0 if `xgsel' + `xbout' <= 0;

   };

   



   /* This section calculates the conditional marginal effect at means */



   tempvar mxgsel mxbout mea me1 men1 mme1 mmen1 megam megama mebet nmxbout

   	mea meb mx mxx ma mb mea1 mea2 mg1 mb1 nmb1;

   sum `xgsel'; 

   local mxgsel = _result(3);     /* creates mean x * gamma */

   sum `xbout';

   local mxbout = _result(3);     /* creates mean x * beta */

   local pnxb = normd(`mxbout');  /* calculates density of mean x * beta */  

   local cnxg = norm(`mxgsel');   /* calculates cdf of mean x * gamma */

   local mxa = `pnxb'/`cnxg';     

   matrix `mxx' = (`cnxg');

   matrix `mx' = (`mxa');        

   matrix `megam' = `coefg'*`mx';  /* calculates marginal effect at means for select var in outcome equation */

   matrix `mebet' = `coefb'*`mx';  /* calculates marginal effect at means for outcome var in outcome equation */ 

   matrix `megama' = `coefg'*`mxx';  /* calculates marginal effect at means for selection equation */

   /* this section tests whether conditional means satisfy requirements */


   matrix `nmxbout' = -1*`mxbout';    /* - (mean x * beta) */

   scalar `mb1' = `mxbout';

   scalar `mg1' = `mxgsel';


   noi disp "  ";

   noi disp "Conditional marginal effect at the means (Selection):";

   mat mean_sel = `megama'[1, 1..`pm1'];

   noi mat list mean_sel;

   if `corr' == -1 & `mb1'+`mg1' >= 0 {;

      noi disp "  ";

      noi disp "Conditional marginal effect at the means (Outcome):";

      mat mean_out = `mebet'[1, 1..`pm1'];

      noi mat list mean_out; 

   };

   if `corr' == -1 &  `mb1'+`mg1' < 0 {;

      noi disp "  ";

      noi disp "Conditional marginal effect at the means:";

      noi disp "0. See help file."; 

   };   

   if `corr' == 1 & `mg1' >= `mb1' {;

      noi disp "  ";

      noi disp "Conditional marginal effect at the means (Outcome):";

      mat mean_out = `mebet'[1,1..`pm1'];

      noi mat list mean_out;  

   };

   if `corr' == 1 & `mg1' < `mb1' {;

      noi disp "  ";

      noi disp "Conditional marginal effect at the means:";

      noi disp "0. See help file."; 

   };

   

   /* Calculating Standard Errors Using Delta Method */

   /* Calculating G */

   if `se' == 1 {;     /* Standard errors will only be calculated if se = 1 */ 

      local i = 1;

      while `i' < `p' {;    /* loop through each  coefficient */

        local j = `i' + `p';   /* generates index for outcome coeffs */

        tempvar g`i' g`j' xx`i';

        local xn = "`1'";

        gen `xx`i'' = `xn';   /* separates the covariates */

        macro shift 1;        /* sets up xn matrix so that the next covariate will be labelled `1' */

        if `corr' == 1 {

           gen `g`i'' =

           (-`xx`i''*normd(`xgsel')*norm(`xbout'))/((norm(`xgsel'))^2);  /* calc G select coeffs on covariates */

        };

        if `corr' == -1 {

           gen `g`i'' =

           (-`xx`i''*normd(`xgsel')*(1-norm(`xbout')))/((norm(`xgsel'))^2);  /* calc G select coeffs on covariates */

        };

        gen `g`j'' = (`xx`i''*normd(`xgsel'))/(norm(`xgsel'));        /* calc G outcome coeffs on covariates */

        local i = `i' + 1;

      };  

      tempvar g`p' g`p2';

      if `corr' == 1 {

         gen `g`p'' = (normd(`xgsel')*norm(`xbout'))/((norm(`xgsel'))^2); /* calc G for selection const */

      };

      if `corr' == -1 {

         gen `g`p'' = (normd(`xgsel')*(1-norm(`xbout')))/((norm(`xgsel'))^2); /* calc G for selection const */

      };

      gen `g`p2'' = (normd(`xgsel'))/(norm(`xgsel'));                  /* calc G for outcome const */

  

      local i = `p2';

      while `i' > 0 {

         order `g`i'';         /* reordering Gs to make vector with constants */

         replace `g`i'' = 0 if `g`i'' == .;

         local i = `i' - 1;

      };

      sum `pcond';

      local n = _result(1);

      local i = 1;

      gen se`psel' = .;

      gen se`pcond' = .;

      tempvar G Gp gvcvg gvcvgp gx gp;

      while `i' < `n'+1 {;     /* loop through each pred prob */

        mkmat `g1'-`g`p2'', matrix(`G'), if _n == `i';  /* makes G vec for all coeff */

        mkmat `g1'-`g`p'', matrix(`Gp'), if _n == `i';  /* makes G vec for selection coefs */

        mat `gvcvg' = `G'*`VCV'*`G'';                   /* calc G VCV G' for all coefs */

        mat `gvcvgp' = `Gp'*`VCVp'*`Gp'';               /* calc G VCV G' for select coefs */

        scalar gx = `gvcvg'[1,1];                       /* G VCV G' should be a scalar */

        scalar gp = `gvcvgp'[1,1];

        replace se`pcond' = sqrt(gx) in `i';              /* sqrt of G VCV G' is the standard error */

        replace se`psel' = sqrt(gp) in `i';

        local i = `i' + 1;     

      };

      if `corr' == 1 {
     
        replace se`pcond' = . if `pcond' == 1;
    
      };

      if `corr' == -1 {
     
        replace se`pcond' = . if `pcond' == 0;
    
      };

   };

 };

   

end;
